/*******************************************************************************
 * Servo_amk.c
 *
 * Autor: Cludio Lrios
 *
 * Data:18/07/2016
 *
 * Objetivo: Promover o engate de reduo do Amarok baseado na escolha feita
 * atravs de uma chave seletora de 3 posies (42, 44 e 44 reduzida).
 *
 * Use CCS C Compiler 
 * Este arquivo  parte integrante do blog Lrios.tecnologia.ws
 ******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//config

//bits
#bit    out_a = 0x5.5   //pino 2  //sada A do motor DC  via ponte H
#bit    out_b = 0x5.4   //pino 3  //sada B do motor DC  via ponte H
#bit    led   = 0x5.2   //pino 5  //led indicador de engate
#bit    GO_DONE = 0X1F.1 //flag de inicio de converso do AD

//bytes
#byte   GPIO   = 0X05
#byte   TRISIO = 0X85
#byte   CMCON  = 0X19
#byte   TMR1L  = 0X0E
#byte   TMR1H  = 0X0F
#byte   ANSEL  = 0X9F
#byte   T1CON  = 0X10
#byte   ADCON0  = 0X1F
#byte   OPTION_REG  = 0X81
#byte   ADRESH  = 0X1E
#byte   ADRESL = 0X9E

//defines
#define  zona_morta 32    //zona desligada
#define FREQ_PISCADA 5

//variaveis globais
unsigned int16  r_pot,r_sinal;//int16
unsigned int8  cont=1;//int8
int1 f_led;//int1


/*******************************************************************************
                          ROTINA MAIN
 ******************************************************************************/

void main() {
    GPIO=0;//desliga saidas
    TRISIO= 0B001011;//ajusta direo dos pinos
    CMCON=7;//desliga comparador interno
    delay_ms(500);

/*******************************************************************************
                   LOOP PRINCIPAL DE REPETIO
 ******************************************************************************/
    for(;;){
 /*******************************************************************************
                   LE POTENCIOMETRO DE RECOPIA
 ******************************************************************************/   
    
        ANSEL= 0B00010010; //clock /8 an1 analogico
        ADCON0= 0B10000101;// canal 1 AN1 pino 6 justificado a direita
        delay_us(20);//delay
        GO_DONE=1;//inicia uma converso AD
        while(GO_DONE);//aguarda trmino da converso
        r_pot= ((int16)ADRESH<<8)|ADRESL;//salva valor do potenciometro de recpia
/*******************************************************************************
                   LE VREF DA CHAVE SELETORA
 ******************************************************************************/  

        ANSEL= 0B00010001; //clock /8 an0 analogico
        ADCON0= 0B10000001;// canal 0 AN0 pino 7 justificado a direita
        delay_us(20);//delay
        GO_DONE=1;//inicia uma converso AD
        while(GO_DONE);//aguarda trmino da converso
        r_sinal= ((int16)ADRESH<<8)|ADRESL;//salva valor da seletora
 /*******************************************************************************
                  COMPARA LEITURAS
 ******************************************************************************/     

        if(r_sinal >(r_pot+zona_morta)){//compara se maior do que o potencimetro
            out_a=1; out_b=0; f_led=1;
        }
        if(r_sinal <r_pot){    //compara se menor do que o potencimetro
            out_a=0; out_b=1; f_led=1;
        }

        if((r_sinal <=(r_pot+zona_morta))&&(r_sinal >=r_pot)){//compara se est na zona desligada
            out_a=0; out_b=0; f_led=0;
        }

 /*******************************************************************************
         ACIONA LED - PISCA SE FORA DE POSIO E ACENDE SE CORRETO
 ******************************************************************************/        
        
        if(f_led==0){led=1;cont=1;}//correto -mantm led somente ligado
        else{
            if(--cont==0) {cont=FREQ_PISCADA; led=!led; }//incorreto- pisca led
        }

        delay_ms(25);//delay 25 ms

    }


}
